def dragon_curve_letters(n):
	if n == 0:
		return ""
	else:
		prev = dragon_curve_letters(n-1)
		flip = "".join("R" if l == "L" else "L" for l in prev)[::-1]
		return prev + "R" + flip

def cleanup(arr):
	if all(seg == "  " for seg in arr[0]):
		arr = arr[1:]
	if all(seg == "  " for seg in arr[-1]):
		arr = arr[:-1]
	if all(row[0] == "  " for row in arr):
		arr = [row[1:] for row in arr]
	if all(row[-1] == "  " for row in arr):
		arr = [row[:-1] for row in arr]
	return arr

def dragon_curve_string(n):
	if n == 0:
		return "│ "
	letters = dragon_curve_letters(n)
	out = [["  "], ["╵ "]]
	pos = [1, 0]  #y, x
	head = 0
	
	for i, turn in enumerate(letters):
		
		# Enlarge grid
		if pos[0] == len(out) - 1:
			out.append(["  " for i in range(len(out[0]))])
		if pos[1] == len(out[0]) - 1:
			out = [row + ["  "] for row in out]
		if pos[0] == 0:
			out.insert(0, ["  " for i in range(len(out[0]))])
			pos[0] = 1
		if pos[1] == 0:
			out = [["  "] + row for row in out]
			pos[1] = 1
		
		# up	
		if head == 0:
			pos[0] -= 1
			if turn == "R":
				head = 1
				out[pos[0]][pos[1]] = ("┌─" if out[pos[0]][pos[1]] == "  " else "┼─")

			elif turn == "L":
				head = 3
				out[pos[0]][pos[1]] = ("┐ " if out[pos[0]][pos[1]] == "  " else "┼─")
		
		# right
		elif head == 1:
			pos[1] += 1
			if turn == "R":
				head = 2
				out[pos[0]][pos[1]] = ("┐ " if out[pos[0]][pos[1]] == "  " else "┼─")

			elif turn == "L":
				head = 0
				out[pos[0]][pos[1]] = ("┘ " if out[pos[0]][pos[1]] == "  " else "┼─")

		# down
		elif head == 2:
			pos[0] += 1
			if turn == "R":
				head = 3
				out[pos[0]][pos[1]] = ("┘ " if out[pos[0]][pos[1]] == "  " else "┼─")

			elif turn == "L":
				head = 1
				out[pos[0]][pos[1]] = ("└─" if out[pos[0]][pos[1]] == "  " else "┼─")

		# left
		elif head == 3:
			pos[1] -= 1
			if turn == "R":
				head = 0
				out[pos[0]][pos[1]] = ("└─" if out[pos[0]][pos[1]] == "  " else "┼─")

			if turn == "L":
				head = 2
				out[pos[0]][pos[1]] = ("┌─" if out[pos[0]][pos[1]] == "  " else "┼─")

	# out and remove empty rows
	return "\n".join("".join(row) for row in cleanup(out))

if __name__ == '__main__':
	print(dragon_curve_string(7))



def sierpinski_triangle_string(n):
	p = pascal(n)
	l = len(p)
	return "\n".join((l-i)*" "+"".join("◢◣" if e % 2 else "  " for e in row) for i, row in enumerate(p))

def pascal(n):
	p = [[1]]
	for i in range(1, 2**n):
		row = []
		for j in range(1, i):
			row.append(p[-1][j-1]+p[-1][j])
		p.append([1] + row + [1])
	return p

if __name__ == '__main__':
	print(sierpinski_triangle_string(3))



import time
import importlib.util

def l_system(start, rules, n):
	letters = start
	rules = rules.replace("(", "").replace(")", "").replace(" ", "").split(",")
	rules = dict(rule.split("->") for rule in rules)
	for i in range(n):
		letters = "".join(rules[l] if l in rules else l for l in letters)
	return "".join(l for l in letters if l in "+-Ff[]")
	

def cleanup(arr):
	if all(seg == "  " for seg in arr[0]):
		arr = arr[1:]
	if all(seg == "  " for seg in arr[-1]):
		arr = arr[:-1]
	if all(row[0] == "  " for row in arr):
		arr = [row[1:] for row in arr]
	if all(row[-1] == "  " for row in arr):
		arr = [row[:-1] for row in arr]
	return arr

def l_system_string(start, rules, n, start_dir="U"):
	
	letters = l_system(start, rules, n)

	out, pos_stack, pos, comp = init_grid(start_dir)

	for i, turn in enumerate(letters):
		
		out, pos_stack, pos = resize_grid(out, pos_stack, pos)
		
		out, pos_stack, pos, comp = grid_command(out, pos_stack, pos, comp, turn)	
	
	out, pos_stack, pos = resize_grid(out, pos_stack, pos)

	return grid_to_string(out)


def init_grid(start_dir="U"):
	
	grid = [["URDL"]]
	pos = [0, 0]  #y, x
	stack = []
	head = "URDL"
	if start_dir not in "URDL":
		raise ValueError(f"start_dir must be U,R,D or L but was {start_dir}")
	while head[0] != start_dir:
		head = head[1:] + head[0]
	
	return grid, stack, pos, head

def grid_to_string(grid):
	trans = {"URDL": "  ", "URD": "╴ ", "URL": "╷ ", "UDL": "╶─",
			 "RDL": "╵ ", "RL": "│ ", "UD": "──", "UR": "┐ ",
			 "UL": "┌─", "RD": "┘ ", "DL": "└─", "U": "┬─",
			 "R": "┤ ", "D": "┴─", "L": "├─", "": "┼─"}
	return "\n".join("".join(trans[c] for c in row) for row in cleanup(grid))


def resize_grid(grid, stack, pos):

	if pos[0] == len(grid) - 1:
		grid.append(["URDL" for i in range(len(grid[0]))])
	if pos[1] == len(grid[0]) - 1:
		grid = [row + ["URDL"] for row in grid]
	if pos[0] == 0:
		grid.insert(0, ["URDL" for i in range(len(grid[0]))])
		pos[0] = 1
		stack = [[[p[0][0]+1, p[0][1]], p[1]] for p in stack]
	if pos[1] == 0:
		grid = [["URDL"] + row for row in grid]
		pos[1] = 1	
		stack = [[[p[0][0], p[0][1]+1], p[1]] for p in stack]
	return grid, stack, pos


def grid_command(grid, stack, pos, head, cmd):
	
	curr = grid[pos[0]][pos[1]]
	if cmd == "-":
		head = head[1:] + head[0]

	elif cmd == "+":
		head = head[-1] + head[:-1]

	elif cmd in "fF":
		if cmd == "F":
			grid[pos[0]][pos[1]] = curr.replace(head[0], "")
		prev_head = head
		if head[0] == "U":
			pos[0] -= 1
		elif head[0] == "R":
			pos[1] += 1
		elif head[0] == "D":
			pos[0] += 1
		elif head[0] == "L":
			pos[1] -= 1
		if cmd == "F":
			grid[pos[0]][pos[1]] = grid[pos[0]][pos[1]].replace(head[2], "")

	elif cmd == "[":
		stack.append([[pos[0], pos[1]], head])

	elif cmd == "]":
		pos, head = stack.pop()

	return grid, stack, pos, head


def l_system_animate(start, rules, n, start_dir="U", speed=0.3):

	if importlib.util.find_spec('curses') is None:
		raise Exception("To use the explore command the curses package must be installed.")
	
	import curses
	from curses import wrapper
	stdscr = curses.initscr()
	letters = l_system(start, rules, n)
	curses.noecho()
	curses.cbreak()
	stdscr.nodelay(True)
	stdscr.keypad(True)
	
	def main(stdscr):
		nonlocal letters
		nonlocal start_dir
		nonlocal speed

		out, pos_stack, pos, comp = init_grid(start_dir)

		for i, turn in enumerate(letters):
		
			key = stdscr.getch()
			curses.flushinp()
			if key == ord('x') or key == 27:
				return grid_to_string(out)

			rows, cols = stdscr.getmaxyx()
			if len(out) >= rows or len(out[0]) >= cols:
				return grid_to_string(out)
 
			out, pos_stack, pos = resize_grid(out, pos_stack, pos)

			stdscr.addstr(0, 0, grid_to_string(out))
			stdscr.refresh()
			time.sleep(speed)	
			
			out, pos_stack, pos, comp = grid_command(out, pos_stack, pos, comp, turn)
		
		out, pos_stack, pos = resize_grid(out, pos_stack, pos)

		return grid_to_string(out)

	s = wrapper(main)
	curses.endwin()
	return s

if __name__ == '__main__':
	print(l_system_animate("A", "(A->+BF-AFA-FB+),(B->-AF+BFB+FA-)", 5, "U", 0.003))



import importlib.util

def in_mandelbrot(c, max_iter):
	z = 0
	for i in range(max_iter):
		z = z**2 + c
		if abs(z) >= 2:
			return i
	return max_iter

def mandelbrot_string(x, y, x_radius, y_radius, stepsize, max_iter, style="repeating", grid="square"):
	if grid == "rect":
		mset = [[in_mandelbrot(complex(x + dx * stepsize, y - dy * stepsize * 2), max_iter)
				for dx in range(-x_radius, x_radius+1)] for dy in range(-y_radius, y_radius+1)]
		
		if style == "repeating":
			shades = ['░', '▒', '▓', '▒']
			m = len(shades)
			return '\n'.join(''.join(" " if d == 0 else shades[d%m] if d < max_iter else "█" for d in l) for l in mset)

		elif style == "non-repeating":
			shades = [' ', '░', '▒', '▓', '█']
			return '\n'.join(''.join(shades[int(4*d/max_iter)] for d in l) for l in mset)

	elif grid == "square":
		mset = [[in_mandelbrot(complex(x + dx * stepsize, y - dy * stepsize), max_iter)
				for dx in range(-x_radius, x_radius+1)] for dy in range(-y_radius, y_radius+1)]
		if style == "repeating":
			shades = ['░', '▒', '▓', '▒']
			m = len(shades)
			return '\n'.join(''.join("  " if d == 0 else shades[d%m]*2 if d < max_iter else "██" for d in l) for l in mset)

		elif style == "non-repeating":
			shades = [' ', '░', '▒', '▓', '█']
			return '\n'.join(''.join(shades[int(4*d/max_iter)]*2 for d in l) for l in mset)

def mandelbrot_explore(x, y, x_radius, y_radius, stepsize, max_iter, style="repeating", grid="rect"):

	if importlib.util.find_spec('curses') is None:
		raise Exception("To use the explore command the curses package must be installed.")
	import curses
	from curses import wrapper
	stdscr = curses.initscr()
	curses.mousemask(1)
	curses.noecho()
	curses.cbreak()
	stdscr.keypad(True)
	s = stepsize
	i = max_iter

	def main(stdscr):
		nonlocal x
		nonlocal y
		nonlocal x_radius
		nonlocal y_radius
		nonlocal s
		nonlocal style
		nonlocal grid
		nonlocal i
		styles = ["non-repeating", "repeating"]
		si = 0 if style == "non-repeating" else 1
 	
		while True:
			rows, cols = stdscr.getmaxyx()
			y_rad = min(y_radius, rows//2-1)
			x_rad = min(x_radius, cols//(2 if grid == "rect" else 4)-1)
			stdscr.addstr(0, 0, mandelbrot_string(x, y, x_rad, y_rad, s, i, styles[si], grid))
			#stdscr.refresh()
			key = stdscr.getch()
			curses.flushinp()
			if key == curses.KEY_RIGHT:
				x += 5*s
			elif key == curses.KEY_LEFT:
				x -= 5*s
			elif key == curses.KEY_UP:
				y += 5*s
			elif key == curses.KEY_DOWN:
				y -= 5*s
			elif key == curses.KEY_NPAGE:
				s /= 2
			elif key == curses.KEY_PPAGE:
				s *= 2
			elif key == ord(','):
				i -= 1
				i = max(i,1)
			elif key == ord('.'):
				i += 1
			elif key == ord('-'):
				si = 1 - si
			elif key == curses.KEY_MOUSE:
				_, mx, my, _, _ = curses.getmouse()
				x += ((mx if grid == "rect" else mx/2) - x_rad)*s
				y += (-my + y_rad)*(2*s if grid == "rect" else s)
			elif key == 27 or key == ord('x'):
				break
			elif key == ord('h'):
				stdscr.addstr(0, 0, ("CONTROLS:\n\nARROWS       : navigate complex plane\nPAGE-UP/DOWN : zoom in/out\n',' and '.'  :")+
									(" change max-iterations\n'-'          : change style\nESC and 'x'  : leave explore mode and print to screen.\n\n")+
									(f"CURRENT COMMAND:\nfrascii mandelbrot -x {x} -y {y} -x_radius {x_rad} -y_radius {y_rad} -stepsize {s}")+
									(f" -max_iter {i} -style {styles[si]} -grid {grid}"))
				curses.flushinp()
				while stdscr.getch() not in (ord('h'), ord('x'), 27):
					pass

		return x, y, x_rad, y_rad, s, i, styles[si], grid

	args = wrapper(main)
	curses.endwin()
	return args

if __name__ == '__main__':
	mandelbrot_explore(x=-0.8, y=0, x_radius=4000, y_radius=4000, stepsize=0.05, max_iter=20)



def rot90(arr, direction):
	m = len(arr)
	n = len(arr[0])
	if direction == 'cw':
		return [[arr[m-1-j][i] for j in range(m)] for i in range(n)]
	elif direction == 'ccw':
		return [[arr[j][n-1-i] for j in range(m)] for i in range(n)]

def flipud(arr):
	return [row[::-1] for row in arr]

def catx(arr1, arr2):
	return [r1 + r2 for r1, r2 in zip(arr1, arr2)]

def caty(arr1, arr2):
	return arr1 + arr2

def add_value(arr, value):
    return [[elem + value for elem in row] for row in arr]

def hilbert_curve(n):
	if n == 1:
		return [[1, 2], [0, 3]]
	h = hilbert_curve(n-1)
	add = 4 ** (n-1)
	h_up = catx(add_value(h, add), add_value(h, 2*add))
	h_down = catx(flipud(rot90(h, 'ccw')), add_value(flipud(rot90(h, 'cw')), 3*add))
	h = caty(h_up, h_down)
	return h

def hilbert_curve_string(iterations):
	arr = hilbert_curve(iterations)
	out = ""
	n = len(arr)
	for i in range(n):
		for j in range(n):
			if i != 0 and i != n-1 and abs(arr[i-1][j] - arr[i][j]) == 1 and abs(arr[i+1][j] - arr[i][j]) == 1:
				out += "│ "
			elif j != 0 and j != n-1 and abs(arr[i][j-1] - arr[i][j]) == 1 and abs(arr[i][j+1] - arr[i][j]) == 1:
				out += "──"
			elif j != 0 and i != n-1 and abs(arr[i][j-1] - arr[i][j]) == 1 and abs(arr[i+1][j] - arr[i][j]) == 1:
				out += "┐ "
			elif j != 0 and i != 0 and abs(arr[i][j-1] - arr[i][j]) == 1 and abs(arr[i-1][j] - arr[i][j]) == 1:
				out += "┘ "
			elif i != 0 and j != n-1 and abs(arr[i-1][j] - arr[i][j]) == 1 and abs(arr[i][j+1] - arr[i][j]) == 1:
				out += "└─"
			elif i != n-1 and j != n-1 and abs(arr[i+1][j] - arr[i][j]) == 1 and abs(arr[i][j+1] - arr[i][j]) == 1:
				out += "┌─"
			else:
				out += "│ " if iterations % 2 else "──"
			if j == n-1:
				out = out[:-1] + "\n"
	return out[:-1]

if __name__ == '__main__':
	print(hilbert_curve_string(3))



from setuptools import setup, find_packages
from frascii import __version__

def readme():
    with open('README.md', 'r', encoding="utf-8") as f:
        content = f.read()
    return content

setup(	
		name='frascii', version=__version__,
		url='https://github.com/LuggiStruggi/Frascii.git',
		author='Lukas König',
		author_email='lukasmkoenig@gmx.net',
		packages=find_packages(),
		long_description=readme(),
    	long_description_content_type='text/markdown',
		entry_points={'console_scripts': ['frascii=frascii.commands:cmd_main']},
		decription="DESCRIPTION"
	)

logo = f"""        FRASCII version {__version__} installed.

                            █                      
                          ██                       
                       █ █████                     
                      ██████████                   
                 ███ ███████████                   
         ██████████████████████                    
                 ███ ███████████                   
                      ██████████                   
                       █ █████                     
                          ██                       
                            █
"""

print(logo)



def cantor_string(n):
	out = ["▄"*3**n]
	while n >= 1:
		len_old = 3**n
		len_new = 3**(n-1)
		new = out[-1].replace("▄"*len_old, "▄"*len_new + " "*len_new + "▄"*len_new)
		out.append(new)
		n -= 1
	return "\n".join("".join(row) for row in out)

if __name__ == '__main__':
	print(cantor_string(2))
